#!/usr/local/bin/perl -w 
#########################################################################
# urlchk.pl - URLs auf Vernderungen hin berprfen 
#########################################################################
# urlchk.pl [-v] [-once] URL
#   -v:    verbose
#   -once: nur einmal tglich
#########################################################################
                                   
$pfile = "$ENV{HOME}/data/urlchk"; # Pfad fr Persistenz-Datei

use LWP::UserAgent;                # WWW Zugriffe
use GDBM_File;                     # Persistenter Hash
use Fcntl;                         # O_CREAT, O_RDWR usw.

$| = 1;                            # Ausgaben nicht puffern

                                   # Persistenten Hash ffnen
tie(%MEM, GDBM_File, $pfile, O_CREAT|O_RDWR, 0644) || 
      die "Cannot open $pfile";
  
                                   # Kommandozeilenparameter analysieren
$verbose    = 0;                   # Ausgaben erwnscht?
$once_a_day = 0;                   # Nur einmal tglich?
@ARGV = grep { !(/^-v$/ && ($verbose = 1)) } @ARGV;
@ARGV = grep { !(/^-once$/ && ($once_a_day = 1)) } @ARGV;

                                   # Nur einmal tglich
if($once_a_day && $MEM{'julday'} == (localtime(time))[7]) {
  print "No more checks today\n" if $verbose;
  exit 1;
}
                                   # Datum speichern
$MEM{'julday'} = (localtime(time))[7]; 


while($url = shift @ARGV) {        # URLs abklappern

  print "Checking $url ... " if $verbose;

  $ua = LWP::UserAgent->new();     # UserAgent erzeugen
  
                                   # URL prfen
  die "Invalid URL" unless $ua->is_protocol_supported($url =~ /^(.+?):/);
  
                                   # URL festlegen
  $request = HTTP::Request->new('HEAD', "$url");
  
                                   # Netzzugriff ausfhren
  $response = $ua->request($request) || die "Request failed: $url";

                                   # Fehlerprfung
  die "Cannot get $url" unless $response->is_success;
      
                                   # Zeitstempel holen und vergleichen
  if(defined($dat = $response->header('Last-Modified'))) {
    result($url, $dat ne $MEM{"$url.dat"});
    $MEM{"$url.dat"} = $dat;

  } else {
                                   # Kein Zeitstempel - Lnge ermitteln
    unless(defined($len = $response->header('Content-Length'))) {
                                   # Kein Lngenfeld im Header - 
                                   # Dokument holen und Checksumme bestimmen
      $request  = HTTP::Request->new('GET', "$url");
      $response = $ua->request($request) || die "Request failed: $url";
      die "Cannot get $url" unless $response->is_success;
      $len = unpack("%16C*", $response->content());
    }

    result($url, $len ne $MEM{"$url.len"});
    $MEM{"$url.len"} = $len;
  }
}

untie(%MEM);                       # Persistenten Hash schlieen


######################################################################### 
sub result {                       # Ergebnis ausgeben
#########################################################################
    my $url     = shift;
    my $changed = shift;

    if($changed) {
      print "$url " unless $verbose;
      print "changed.\n";
    } else {
      print "unchanged.\n" if $verbose;
    }
}
